package org.jvisu.base;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.comm.var.*;
import org.jvisu.comm.*;
import org.jvisu.panel.*;

/** Abstract base class of JVisu visual components.
 *
 *  <p>$Id: JVBean.java 302 2005-10-16 10:43:06Z ero $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 First implementation </li>
 *  </ul>
 *  <p>
 *      This is the base class for all JVisu visual components <br>
 *      It provides <br>
 *      - common properties<br>
 *      - communication functions <br>
 *      - frame drawing<br>
 *  </p>
 *  <p> The derived classes utilize the basic functions. They contain the
 *      type dependend code for displaying the data contents. </p>
 *
 * @author  Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 * @since   1.0
 */
public abstract class JVBean extends JPanel implements JVVarListener, PropertyChangeListener, JVBaseMembers {
    
    /** Communication instance configuration data */
    private JVCommCfg               commcfg;
    /** Holds value of property frameColor. */
    protected Color                 frameColor;
    /** Maximum updaterate for this component */
    protected int                   updaterate = 1000;
    /** Holds value of property aligncontent. */
    protected int                   aligncontent = JVAlignment.JVISU_ALIGNCENTER;
    /** Holds value of property readonly */
    protected boolean               readonly = true;
    /** Holds value of property editEnabled. */
    protected boolean               editEnabled;
    /** Storage for JVVars configurations , which are used for display purposes */
    protected ArrayList             vars;
    /** Base address for file access */
    protected String                base;
    /** Position of this component */
    protected Point                 designpos;
    /** Size of this component */
    protected Dimension             designsize;
    /** Zoom support */
    protected double                zoomfactor = 1.0;
    /** The fontsize designed for this component */
    protected int                   fontsize;
    /** The designmodel for this component */
    protected String                designmodel;
    
    /** Creates new VisuBean */
    public JVBean() {
        vars = new ArrayList();
        commcfg = new JVCommCfg();
        designpos = new Point(0, 0);
        designsize = new Dimension (50,50);
        designmodel = getClass().getName();
        fontsize = getFont().getSize();
        addPropertyChangeListener(this);
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        if (evt.getSource() == this) {
            String key = evt.getPropertyName();
            /** Own events */
            if (key.compareTo("server") == 0) {
                JVCommCfg cfg = new JVCommCfg(commcfg);
                cfg.setServer((String) evt.getNewValue());
                UpdateCommCfg(cfg);
                return;
            }
                
            if (key.compareTo("commtype") == 0) {
                JVCommCfg cfg = new JVCommCfg(commcfg);
                cfg.setType((String) evt.getNewValue());
                UpdateCommCfg(cfg);
                return;
            }
                
            if (key.compareTo("port") == 0) {
                JVCommCfg cfg = new JVCommCfg(commcfg);
                cfg.setPort(((Integer) evt.getNewValue()).intValue());
                UpdateCommCfg(cfg);
                return;
            }
            if (key.equals("designsize") || key.equals("zoomfactor")) {
                recalcSize();
            }
        } 
    }
        
    /** Called by the parent component, when the component is
     *  added.
     *  Startup component
     */
    public void addNotify() {
        /* Let the superclass do its magic */
        super.addNotify();
        /* Call the derived components startup method */
        StartUp();
        /* Initialize communication */
        StartComm();
        firePropertyChange("designpos", null, designpos);
    }
    
    /** Called by the parent component, when the component is
     *  removed.
     *  Shut down the component and free resources
     */
    public void removeNotify() {
        /* Let the superclass do its magic */
        super.removeNotify();
        /* Call the derived components shutdown method */
        Shutdown();
        /* Free communication resources */
        StopComm();
    }
    
    /** Start communication
     * Register the variables in a communication instance, if possible. 
     */
    private void StartComm() {
        JVComm.RegisterVariables(commcfg, (JVVarCfg[])vars.toArray(new JVVarCfg[vars.size()]), this);
    }

    /** Stop communication
     *  Unregister the variables in a communication instance, if possible. 
     */
    private void StopComm() {
        JVComm.UnRegisterVariables(commcfg, (JVVarCfg[])vars.toArray(new JVVarCfg[vars.size()]), this);
    }
    
    /** Update communication instance data
     */
    private void UpdateCommCfg(JVCommCfg newcfg) {
        JVComm.UpdateCommCfg(commcfg, newcfg, (JVVarCfg[])vars.toArray(new JVVarCfg[vars.size()]), this);
        commcfg = newcfg;
    }

    /** Abstract implementation of JVVarListener */
    public abstract void VarChanged(JVVar var);
    
    /** Startup of the component. Called when throws component is
     *  added to a parent component.
     */
    public abstract void StartUp();
    
    /** Shutdown of the component Called when throws component is
     *  removed from a parent component.
     */
    public abstract void Shutdown();
    
    /** Update the screen image of the component
     *  Update is neccecray to reduce flickering for application
     *  triggered repaint()
     * @param g Graphics context
    public void update(Graphics g) {
        paint(g);
    }
     */
    
    /** Add a variable configuration to the variable config array
     *  @param cfg configuration for this variable
     */
    protected void addCommVar (JVVarCfg cfg) {
        vars.add(cfg);
    }

    /** Set a variable configuration
     *  @param idx index of the variable in the variable configuration array
     *  @param cfg the new configuration
     */
    protected void setCommVar (int idx, JVVarCfg cfg) {
        vars.set(idx, cfg);
        if (cfg.getVarInstance() != null)
            VarChanged(cfg.getVarInstance());
    }
    
    /** Set the variable symbol for a registered commvar
     *  @param idx index of the variable in the variable configuration array
     *  @param symbol new symbol for this variable
     */
    protected void setCommVarSymbol(int idx, String symbol) {
        try {
            String sym = getCommVarCfg(idx).getSymbol();
            if (sym == null && symbol == null)
                return;
            if (sym != null && sym.equals(symbol))
                return;
            JVVarCfg cfg = new JVVarCfg(getCommVarCfg(idx));
            cfg.setSymbol(symbol);
            JVComm.UpdateVarCfg(commcfg, getCommVarCfg(idx), cfg, this);
            setCommVar(idx, cfg);
        } catch (NullPointerException e) { }
    }

    protected void setCommVarSize(int idx, int size) {
        try {
            int oldsize = getCommVarCfg(idx).getSize();
            if (size == oldsize)
                return;

            JVVarCfg cfg = new JVVarCfg(getCommVarCfg(idx));
            cfg.setSize(size);

            JVComm.UpdateVarCfg(commcfg, getCommVarCfg(idx), cfg, this);
            setCommVar(idx, cfg);
        } catch (NullPointerException e) { }
    }


    /** Get the symbol of the communication variable
     *  @param idx index of the variable in the variable configuration array
     *  @return the variable symbol of this component
     */
    protected String getCommVarSymbol(int idx) {
        return getCommVarCfg(idx).getSymbol();  
    }
    
    /** Get variable configuration 
     *  @param  idx index of the variable in the config array
     *  @return variable configuration or null
     */
    protected JVVarCfg getCommVarCfg (int idx) {
        if (idx < vars.size())
            return (JVVarCfg)vars.get(idx);
        else
            return null;
    }
    
    /** Set the enabled bit for a communication variable
     *  @param idx index of the variable in the config array
     *  @param ena the setting of the enable bit
     */
    protected void setCommVarEnable (int idx, boolean ena) {
        try {
            boolean act = getCommVarCfg(idx).isEnabled();
            if (act == ena)
                return;
            JVVarCfg cfg = new JVVarCfg(getCommVarCfg(idx));
            cfg.setEnabled(ena);
            JVComm.UpdateVarCfg(commcfg, getCommVarCfg(idx), cfg, this);
            setCommVar(idx, cfg);
        } catch (NullPointerException e) { }
    }
    
    /** Modify the content of the communication variable 
     *  @param idx index of the variable in the configuration array
     *  @param obj object to set the variable
     */
    protected void setCommVarValue (int idx, Object obj) {
        try {
            JVVarCfg cfg = getCommVarCfg(idx);
            commcfg.getCommInstance().setVarValue(cfg, obj);
        } catch (NullPointerException e) { }
    }
    
    /** Get the string, which describes the server
     * @return The server address. Can be a hostname or an IP-address
     */
    public String getServer() {
        return commcfg.getServer();
    }
    
    /** Setter for property server.
     * @param server New value of property server.
     */
    public void setServer(String server) {
        String old = commcfg.getServer();
        firePropertyChange("server", old, server);
    }
    
    /** Getter for property commtype.
     * @return Value of property commtype.
     *
     */
    public String getCommtype() {
        return commcfg.getType();
    }
    
    /** Setter for property commtype.
     * @param commtype New value of property commtype.
     */
    public void setCommtype(String commtype) {
        String old = commcfg.getType();
        firePropertyChange("commtype", old, commtype);
    }
    
    /** Getter for property port.
     * @return Value of property port.
     */
    public int getPort() {
        return commcfg.getPort();
    }
    
    /** Setter for property port.
     * @param port New value of property port.
     */
    public void setPort(int port) {
        int old = commcfg.getPort();
        firePropertyChange("port", old, port);
    }

    /** Return the class name of the communication resolver
     * @return resolver class name.
     */
    public String getVarResolver() {
        return commcfg.getResolver();
    }
    
    /** Set the class name of the communication resolver
     * @param resolv new resolver class name.
     */
    public void setVarResolver(String resolv) {
        String old = commcfg.getResolver();
        commcfg.setResolver(resolv);
        firePropertyChange("varResolver", old, resolv);
    }
    
    /** Getter for property frameColor.
     * @return Value of property frameColor.
     */
    public Color getFrameColor() {
        return this.frameColor;
    }
    
    /** Setter for property frameColor.
     *  Change the border type depending on the given value
     *  @param frameColor New value of property frameColor.
     */
    public void setFrameColor(Color frameColor) {
        this.frameColor = frameColor;
        if (frameColor != null)
            setBorder(new javax.swing.border.LineBorder(frameColor));
        else
            setBorder(null);
    }
    
    /** Getter for property updaterate.
     * @return Value of property updaterate.
     */
    public int getUpdaterate() {
        return this.updaterate;
    }
    
    /** Setter for property updaterate.
     * @param updaterate New value of property updaterate.
     */
    public void setUpdaterate(int updaterate) {
        int oldUpdaterate = this.updaterate;
        this.updaterate = updaterate;
        if (oldUpdaterate != updaterate) {
            for (int i = 0; i < vars.size(); i++) {
                try {
                    JVVarCfg cfg = new JVVarCfg(getCommVarCfg(i));
                    cfg.setRate(updaterate);
                    JVComm.UpdateVarCfg(commcfg, getCommVarCfg(i), cfg, this);
                    setCommVar(i, cfg);
                } catch (NullPointerException e) { }
            }
        }
    }
    
    /** Getter for property aligncontent.
     * @return Value of property aligncontent.
     */
    public int getAligncontent() {
        return aligncontent;
    }
    
    /** Setter for property aligncontent.
     * @param val Value of property aligncontent.
     */
    public void setAligncontent(int val) {
        int old = aligncontent;
        aligncontent = val;
        firePropertyChange("aligncontent", old, aligncontent);
    }
    
    /** Getter for property readonly.
     * @return Value of property readonly.
     */
    public boolean isReadonly() {
        return this.readonly;
    }
    
    /** Setter for property readonly.
     * @param readonly New value of property readonly.
     */
    public void setReadonly(boolean readonly) {
        boolean oldReadonly = this.readonly;
        this.readonly = readonly;
        firePropertyChange("readonly", oldReadonly, this.readonly);
    }
    
    /** Getter for property editEnable.
     * @return Value of property editEnable.
     */
    public boolean isEditEnabled() {
        return this.editEnabled;
    }
    
    /** Setter for property editEnable.
     * @param editEnable New value of property editEnable.
     */
    public void setEditEnabled(boolean editEnable) {
        boolean oldEditEnable = this.editEnabled;
        this.editEnabled = editEnable;
        firePropertyChange("editEnabled", new Boolean(oldEditEnable), new Boolean(this.editEnabled));
    }

    /** Get the base address for file access.
     * @return base address for file access 
     */
    public String getBase() {
        return this.base;
    }
    
    /** Set the base address for file access.
     *  Handy parameter for relocatable executables and applets
     *  @param newbase new base address for file access.
     */
    public void setBase(String newbase) {
        String oldbase = base;
        base = newbase;
        firePropertyChange("base", oldbase, base);
    }
    
    /** Getter for property designpos.
     * @return Value of property designpos.
     */
    public Point getDesignpos() {
        return designpos;
    }
    
    /** Setter for property designpos.
     *  @param newpos new position
     */
    public void setDesignpos(Point newpos) {
        Point oldpos = new Point(designpos);
        designpos.setLocation(newpos);
        firePropertyChange("designpos", oldpos, newpos);
    }

    /** get the designmodel for this component 
     * @return designmodel
     */
    public String getDesignModel() {
        return designmodel;
    }
    
    /** set the designmodel for this component 
     * @param model designmodel
     */
    public void setDesignModel(String model) {
        designmodel = model;
    }
    
    /** Get the ability of this component to have subcomponents
     *  This should be overridden by container classes
     * @return true, if subcomponents are allowed
     */
    public boolean isContainer() {
        return false;
    }
    
    /** Return the design size of this component
     * @return design size
     */
    public Dimension getDesignsize() {
        return designsize;
    }
    
    /** Set the design size of this component.
     * @param newpos new designsize
     */
    public void setDesignsize(Dimension newsize) {
        Dimension oldsize = new Dimension (designsize);
        designsize.setSize(newsize);
        firePropertyChange("designsize", oldsize, newsize);
    }

    /** Get the zoom factor of this component
     * @return zoomfactor for this component
     */
    public double getZoomfactor() {
        return zoomfactor;
    }

    /** Set the zoom factor of this component
     * @param zoom new zoomfactor for this component
     */
    public void setZoomfactor(double zoom) {
        if (zoom < 0.25 || zoom > 2.5)
            return;
        double oldzoom = zoomfactor;
        zoomfactor = zoom;
        firePropertyChange("zoomfactor", oldzoom, zoom);
    }
    
    /** Recalc the size depending on zoom factor and designsize 
     */
    private void recalcSize() {
        Dimension d = new Dimension(designsize);
        d.setSize(zoomfactor * d.getWidth(), zoomfactor * d.getHeight());
        this.setPreferredSize(d);
        Font font = super.getFont();
        super.setFont(new Font(font.getName(), font.getStyle(), 
            (int)(zoomfactor * (double) fontsize)));
    }
    
    /** Set the font 
     *  @param font the new font
     */
    public void setDesignFont(Font font) {
        Font old = getFont();
        fontsize = font.getSize();
        super.setFont(new Font(font.getName(), font.getStyle(), 
            (int)(zoomfactor * (double) fontsize)));
    }

    /** Get the font 
     *  @param font the new font
     */
    public Font getDesignFont() {
        Font font = super.getFont();
        return new Font(font.getName(), font.getStyle(), fontsize);
    }
}
