package org.jvisu.base;

import java.awt.*;
import java.text.*;
import java.beans.*;
import javax.swing.*;
import org.jvisu.comm.*;
import org.jvisu.util.param.JVParams;
import org.jvisu.comm.var.*;

/** Visual component to display numerical (double) values
 *
 * $Id: JVDouble.java 275 2004-04-19 08:20:55Z tgl $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *  The purpose of JVDigits is to display a rectangular shape, which contains
 *  the numerical representation of a variable. The representation can be
 *  customized by various methods.
 *  The components has a builtin min/max compare function, which can be custumized by
 *  absolute values or variable values, which are retrieved from the communication server
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public abstract class JVDouble extends JVBean implements JVVarListener {
    
    /** The variable value */
    private double              varval;
    
    /** Value to display */
    protected double            actval;
    
    /** minimum value to which the display value is compared.
     * If the display value is less than minimun, then the alarmcolor is set as
     * background color.
     */
    protected double            minval = 0;
    
    /** maximum value to which the display value is compared.
     * If the display value is greater than maximun, then the alarmcolor is set as
     * background color.
     */
    protected double            maxval = 1000;

    /** Offset for scaling */
    protected double            offset = 0;
    /** Scaling factor */
    protected double            scalefactor = 1;
    
    /** Selects, if minval is controlled by a variable*/
    private boolean             minvalByVariable;
    /** Selects, if maxval is controlled by a variable*/
    private boolean             maxvalByVariable;
    /** Numberformat variable to provide formatting of the display value */    
    protected NumberFormat      numFormat;
    
    /** Create a new instance of VisuDigits
     */
    public JVDouble() {
        // Build commvariables for act, min, max
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, 0));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, 1));
        addCommVar(new JVVarCfg(null, updaterate, JVVar.T_DOUBLE, 0, 2));
        // Instantiate a new Numberformat
        numFormat = NumberFormat.getInstance(java.util.Locale.US);
        numFormat.setMinimumIntegerDigits(1);
        numFormat.setMaximumIntegerDigits(10);
        numFormat.setMinimumFractionDigits(0);
        numFormat.setMaximumFractionDigits(0);
    }
    
    /** Subclasses have to implement a start function
     */
    public void StartUp() {
    }
    
    /** Subclasses have to implement a stop function
     */
    public void Shutdown() {
    }
    
    /** Get maximum number of integer digits
     *  @return maximum number of integer digits
     */
    public int getIntDigits() {
         return numFormat.getMaximumIntegerDigits();
    }

    /** Set maximum number of integer digits
     *  @param dig maximum number of integer digits
     */
    public void setIntDigits(int dig) {
        int old = numFormat.getMaximumIntegerDigits();
        numFormat.setMaximumIntegerDigits(dig);
        firePropertyChange("intDigits", old, dig);
    }

    /** Get maximum number of fractional digits
     *  @return maximum number of fractional digits
     */
    public int getFracDigits() {
         return numFormat.getMaximumFractionDigits();
    }
    
    /** Set maximum number of fractional digits
     *  @param dig maximum number of fractional digits
     */
    public void setFracDigits(int dig) {
        int old = numFormat.getMaximumFractionDigits();
        numFormat.setMaximumFractionDigits(dig);
        firePropertyChange("fracDigits", old, dig);
    }

    /** Get minimum number of fractional digits
     *  @return minimum number of fractional digits
     */
    public int getMinFracDigits() {
         return numFormat.getMinimumFractionDigits();
    }
    
    /** Set maximum number of fractional digits
     *  @param dig maximum number of fractional digits
     */
    public void setMinFracDigits(int dig) {
        int old = numFormat.getMinimumFractionDigits();
        numFormat.setMinimumFractionDigits(dig);
        firePropertyChange("fracDigits", old, dig);
    }

    /** Set variable value, send it to server, if possible
     */
    public void setValue(double val) {
        setCommVarValue(0, new Double((val - offset) / scalefactor));
    }
    
    /** Get the symbol of the communication variable
     * @return the variable symbol of this component
     */
    public String getCommVar() {
        return getCommVarSymbol(0);
    }
    
    /** Set the symbol of the communication variable
     * @param symbol the variable symbol of this component
     */
    public void setCommVar(String symbol) {
        setCommVarSymbol(0, symbol);
        VarChanged(null);
    }
    
    /** Return setting, if minval is controlled by a variable
     *  @return true, if minval is controlled by a variable
    */
    public boolean getMinvalByVariable () {
        return minvalByVariable;
    }

    /** Control, if minval is controlled by a variable
     *  @param val true, if minval is controlled by a variable
    */
    public void setMinvalByVariable (boolean val) {
        minvalByVariable = val;
        setCommVarEnable(1, val);
        VarChanged(null);
    }

    /** Return setting, if maxval is controlled by a variable
     *  @return true, if maxval is controlled by a variable
    */
    public boolean getMaxvalByVariable () {
        return maxvalByVariable;
    }

    /** Control, if minval is controlled by a variable
     *  @param val true, if minval is controlled by a variable
    */
    public void setMaxvalByVariable (boolean val) {
        maxvalByVariable = val;
        setCommVarEnable(2, val);
        VarChanged(null);
    }

    /** Get the content of the minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the minimum value.
     */
    public String getMinVar() {
        if (!minvalByVariable)
            return Double.toString(minval);
        else
            return getCommVarSymbol(1);
    }
    
    /** Setter for property minVar.
     * @param minVar New value of property maxVar.
     */
    public void setMinVar(String minVar) {
        if (minvalByVariable)
            setCommVarSymbol(2, minVar);
        else
            setMinVal(JVParams.StringToDouble(minVar, minval));
    }
    
    /** Get the content of the minimum value.
     *  This can be a numerical value or a variable symbol
     *  @return the minimum value.
     */
    public String getMaxVar() {
        if (!maxvalByVariable)
            return Double.toString(maxval);
        else
            return getCommVarSymbol(2);
    }
    
    /** Setter for property maxVar.
     * @param maxVar New value of property maxVar.
     */
    public void setMaxVar(String maxVar) {
        if (maxvalByVariable)
            setCommVarSymbol(2, maxVar);
        else
            setMaxVal(JVParams.StringToDouble(maxVar, maxval));
    }

    /** return the actual value */
    public double getActVal() {
        return actval;
    }

    /** set the minimum value 
     *  @param min the new minimum value
     */
    private void setMinVal(double min) {
         double old = minval;
         minval = min;
         firePropertyChange("minval", old, minval);
    }

    /** set the maximum value 
     *  @param max the new maximum value
     */
    private void setMaxVal(double max) {
         double old = maxval;
         maxval = max;
         firePropertyChange("maxval", old, maxval);
    }

    /** get the offset value for scaling
     *  @return the offset value
     */
    private double getOffset() {
        return offset;
    }

    /** set the offset value for scaling
     *  @param offs the new offset value
     */
    private void setOffset(double offs) {
        offset = offs;
        actval = offset + scalefactor * varval;
        repaint();
    }

    /** get the scale factor for scaling
     *  @return the scalefactor
     */
    private double getScalefactor() {
        return scalefactor;
    }

    /** set the scale factor
     *  @param fac the new scalefactor
     */
    private void setScalefactor(double fac) {
        if (fac == 0)
            return;
        scalefactor = fac;
        actval = offset + scalefactor * varval;
        repaint();
    }
    
    /** Informes the owner, that the variable value has been changed
     * @param var Variable to display
     */
    public void VarChanged(JVVar var) {
        if (var != null) {
            if (getCommVarCfg(0).getSymbol() != null &&
            getCommVarCfg(0).getSymbol().equals(var.getVarCfg().getSymbol())) {
                varval = ((JVVarDouble)var).getVarValue();
                actval = offset + scalefactor * varval;
            } else if (getCommVarCfg(1).getSymbol() != null &&
            getCommVarCfg(1).getSymbol().equals(var.getVarCfg().getSymbol())) {
                setMinVal(((JVVarDouble)var).getVarValue());
            } else if (getCommVarCfg(2).getSymbol() != null &&
            getCommVarCfg(2).getSymbol().equals(var.getVarCfg().getSymbol())) {
                setMaxVal(((JVVarDouble)var).getVarValue());
            }
        }
        repaint();
    }
}
