package org.jvisu.base;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.net.*;
import java.text.*;
import java.beans.*;
import javax.swing.*;
import javax.swing.border.*;
import org.jvisu.panel.*;
import org.jvisu.comm.*;
import org.jvisu.util.param.JVParams;
import org.jvisu.comm.var.*;

/** Visual component to display an analog gauge
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 * <p>
 *
 * <p>
 * $Id: JVScales.java 275 2004-04-19 08:20:55Z tgl $
 * </p>
 * <p>
 * Change Log:
 * </p>
 * <ul>
 * <li> 1.0 First implementation
 * </ul>
 * <p>
 * </p>
 *  Abstract base class for all kind of scales,
 *  like gauges, bars and sliders. It provides the common
 *  properties.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public abstract class JVScales extends JVDouble {
    
    /** Constant to calc log10 */
    public static final double  LOG10_FAC = 1 / Math.log(10);
    /** color for filling the scale */
    protected Color             fillColor = Color.yellow;
    /** color for filling the scale under alarm condition*/
    protected Color             alarmColor = Color.red;
    /** Number of major tics */
    protected int               majortics = 5;
    /** Number of minor tics */
    protected int               minortics = 0;
    /** Alarm below n% of the scale */
    protected int               minalarm = 10;
    /** Alarm above n% of the scale */
    protected int               maxalarm = 90;
    /** Logarithmic scaling */
    protected boolean           isLog;
    /** Draw the backgrund with a border*/
    protected boolean           drawBorder;
    /** Recalculate  */
    protected boolean           recalc = true;
    /** border, if the backgorund should be drawn with border */
    protected BevelBorder       border = new BevelBorder(BevelBorder.LOWERED);
    
    /** Create a new instance of JVScales
     */
    public JVScales() {
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        /* recalc and repaint */
        if (key.equals("foreground") || 
            key.equals("font") ||
            key.equals("preferredSize") ||
            key.equals("minortics") ||
            key.equals("majortics") ||
            key.equals("logarithmic") ||
            key.equals("minval") ||
            key.equals("maxval") ||
            key.equals("fracDigits") ||
            key.equals("intDigits")) {
            recalc = true;
            repaint();
        }
    }
    
    /** Select the color depending on the given values
     *  @param cmp is the value, which should be compared
     *  @param tot is the total extent of the scale, which
     *          is used to calculate the min and max treshholds
     *  @return fillColor, if the value is > min and < max, else alarmColor
     */
    protected Color getColorForVal(double cmp, double tot) {
        
        if (cmp < tot * ((double) minalarm) / 100 ||
                cmp > tot * ((double) maxalarm) / 100)
            return alarmColor;
        else 
            return fillColor;
    }
    
    /** Calculate the position of a measuring value
     *  @param val the value to calc
     *  @param lim a limitation for values below or above the scale.
     *         This parameter is multiplied with the full scale extent
     *         and added / subtracted from the violated limitation
     *         This is used for a gauge display to show, that a value is
     *         outside of the limits. Other components like bars and
     *         sliders should set this to 0, so the value is limited
     *         to either max or minval
     *  @return the position as fraction of the full scale
     */
    protected double calcPosition (double val, double lim) {
        if (maxval == minval)
            return 0;
        if (val > maxval)
            val = lim * (maxval-minval) + maxval;
        if (val < minval)
            val = minval - lim * (maxval-minval);
        double f;
        if (isLog) {
            if (val < minval)
                return (val - minval) / (maxval - minval);
            double max = LOG10_FAC * Math.log (maxval);
            double min = LOG10_FAC * Math.log (minval);
            f = (LOG10_FAC * Math.log(val) - min) / (max - min);
        } else {
            f = (val - minval) / (maxval - minval);
        }
        return f;    
    }

    /** Calculate the position of a measuring value
     *  @param pos the value to calc
     *  @param len the lenght of the bar
     *  @return the value for that position
     */
    protected double calcValAtPosition (int pos, int len) throws ArithmeticException {
        if (maxval <= minval)
            throw new ArithmeticException("JVScales: Max <= Min");
        double f =((double) pos) / ((double) len) ;
        if (isLog) {
            if (minval <= 0)
                throw new ArithmeticException("JVScales: Min <= 0");
            return minval * Math.pow ((maxval/minval), f);
        } else {
            return minval + f * (maxval - minval);
        }    
    }    
    /** Calculate the tick position for a major tick
     *  @param major major tick number
     *  @param invers if true, the position is negated
     *  @return the tick position as fraction of the full scale
     */
    protected double calcMajorTick(int major, boolean invers) {
        double f = (double) major / (double) majortics;
        return invers ? -f : f;    
    }

    /** Calculate the tick text for a major tick
     *  @param int major tick number
     *  @return the description text for this tick
     */
    protected String calcMajorText(int tic) {
        double f = (double) tic / (double) majortics;
        if (isLog) {
            if (minval <= 0 || maxval < minval)
                return "";
            double max = LOG10_FAC * Math.log (maxval);
            double min = LOG10_FAC * Math.log (minval);
            f = Math.pow(10, (max - min) * f + min);
        } else {
            f = f * (maxval - minval) + minval;
        }
        return numFormat.format(f);
    }

    /** Calculate the tick position for a minor tick
     *  @param major major tick number
     *  @param minor minor tick number
     *  @param invers if true, the position is negated
     *  @return the tick position as fraction of the full scale
     */
    protected double calcMinorTick(int major, int minor, boolean invers) {
        double f = (double) major / (double) majortics;
        if (isLog) {
            if (minval <= 0 || maxval < minval)
                return 0;
            // calc distance to next major tick
            double dist = LOG10_FAC / (double) majortics;
            f += dist * Math.log(10 * ((double) minor)/ (double) minortics);
        } else {
            f += (double) minor / (double) (majortics * minortics);            
        }            
        return invers ? -f : f;    
    }
    
    /** Get the number of minor tics on the scale
     * @return number of minor tics
     */
    public int getMinortics() {
        return minortics;
    }
    
    /** Set the number of minor tics on the scale
     * @param tics the number of minor tics
     */
    public void setMinortics(int tics) {
        int old = minortics;
        minortics = tics;
        firePropertyChange ("minortics", old, tics);
    }
    
    /** Get the number of major tics on the scale
     * @return number of major tics
     */
    public int getMajortics() {
        return majortics;
    }
    
    /** Set the number of major tics on the scale
     * @param tics the number of major tics
     */
    public void setMajortics(int tics) {
        int old = majortics;
        majortics = tics;
        firePropertyChange ("majortics", old, tics);
    }
    
    /** Get the parameter for logarithmic scaling
     * @return true, if logarithmic scaling is active
     */
    public boolean isLogarithmic() {
        return isLog;
    }

    /** Set the parameter for logarithmic scaling
     * @param log true, if logarithmic scaling is desired
     */
    public void setLogarithmic(boolean log) {
        boolean old = isLog;
        isLog = log;
        firePropertyChange ("logarithmic", old, isLog);
    }
    
    /* The following properties do not affect the 
     * background image. They just affect the drawing of the 
     * current value. So we don't need to repaint 
     * the background image and we fire no propertyChange event
     * A plain repaint is enough.
     */ 
    
    /** Get the fill color for the arc
     * @return the fill color of the arc
     */
    public Color getFillColor() {
        return fillColor;
    }
    
    /** Setter for property fillColor.
     * @param emptyColor New value of property fillColor.
     */
    public void setFillColor(Color color) {
        fillColor = color;
        repaint();
    }
    
    /** Get the alarm color of the scale
     * @return alarm color
     */
    public Color getAlarmColor() {
        return alarmColor;
    }
    
    /** Set the alarm color of the scale
     * @param color the new alarm color
     */
    public void setAlarmColor(Color color) {
        alarmColor = color;
        repaint();
    }

    /** Get the alarm treshhold for min in % of the scale
     * @return alarm treshold min (% of scale)
     */
    public int getMinalarm() {
        return minalarm;
    }
    
    /** Set the alarm treshhold for min in % of the scale
     * @param min alarm treshold min (% of scale)
     */
    public void setMinalarm(int min) {
        minalarm = min;
        repaint();
    }
    
    /** Set the alarm treshhold for max in % of the scale
     * @return alarm treshold max (% of scale)
     */
    public int getMaxalarm() {
        return maxalarm;
    }
    
    /** Set the alarm treshhold for max in % of the scale
     * @param max alarm treshold max (% of scale)
     */
    public void setMaxalarm(int max) {
        maxalarm = max;
        repaint();
    }

    /** Get wether the background has a border
     * @return true, if background has a border
     */
    public boolean isDrawBorderEnabled() {
        return drawBorder;
    }
    
    /** Set wether the background has a border
     * @param ena true, if background has a border
     */
    public void setDrawBorderEnabled(boolean ena) {
        drawBorder = ena;
        repaint();
    }
}
