package org.jvisu.comm;

import java.io.*;
import java.net.*;
import java.nio.*;
import java.nio.channels.*;
import org.jvisu.comm.var.*;

/** JVisu socket based communication
 *
 *  <p>$Id: JVSockComm.java 302 2005-10-16 10:43:06Z ero $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *  
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  <p>This class provides the JVisu socket based communication functions.
 *  </p>
 *  <p><b>TODO:</b> Check all variable related stuff for thread safety
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0 
 * @since JVisu 1.0
*/
public class JVSockComm extends JVComm implements Runnable {

    /** The default port for JVisu socket connections */
    public static final int  JVISU_PORT = 2200;    
    
    // Some real private constants 
    /** Protocol command for registering variables */    
    private static final byte CMD_REGVAR = 1;
    /** Protocol command for unregistering variables */    
    private static final byte CMD_UNREGVAR = 2;
    /** Protocol command for sending variable data */    
    private static final byte CMD_VISU_DATA = 3;
    /** Size of the communication header */    
    private static final int  HEADER_SIZE = 4;
    /** size of the variable header */    
    private static final int  VAR_HEADER_SIZE = 8;
    /** the position of the datagram length information */    
    private static final int  LENGTH_POS = 2;
    
    /** The socket which is hopefully alive */
    private SocketChannel   socket = null;
    /** A thread to manage connect / reconnect ...*/
    private Thread          commThread;
    /** Stay in the thread, as long as this is active */
    private boolean         isActive;
    /** Storage for receive data */
    private ByteBuffer      recBuffer;
    /** Request variable registration with server */
    private boolean         reqRegistration = false;
    /** Index of server registration */
    private int             registeredVars = 0;
    
    /** Creates a new instance of JVCommSock
     */
    public JVSockComm() {
        recBuffer = ByteBuffer.allocate(1024);
        recBuffer.order(java.nio.ByteOrder.LITTLE_ENDIAN);
        //JVComm.setStatusMessage(this, 1, "Created");
    }

    /** Check, if the instance is matching 
     *  @param cfg configuration to compare
     *  @return true, if matching
     */
    public boolean isMatching (JVCommCfg cfg) {
        return (isActive &&
                cfg.getType() != null &&
                cfg.getType().compareTo(this.getClass().getName()) == 0 &&
                cfg.getServer() != null &&
                cfg.getServer().compareTo(commcfg.getServer()) == 0 &&
                cfg.getPort() == commcfg.getPort());
    }

    /** Check, if the instance is reuseable
     *  @param cfg configuration to compare
     *  @return true, if reuseable
     */
    public boolean isReusable (JVCommCfg cfg) {
        return (!isActive &&
                cfg.getType() != null &&
                cfg.getType().compareTo(this.getClass().getName()) == 0);
    }
    
    /** Check, if the given configuration is valid for
     * this communication type
     * @param cfg the configuration to check
     * @return true, if valid
     */
    protected boolean isValid(JVCommCfg cfg) {
        if (cfg.getServer() == null)
            return false;
        try {
            if (InetAddress.getByName(cfg.getServer()) == null)
                throw new UnknownHostException();
            if (cfg.getPort() < 1024)
                throw new Exception();
        } catch (UnknownHostException e) { 
            JVComm.setStatusMessage(this, 1, "Unknown host: " + cfg.getServer());
            return false; 
        } catch (Exception e) {
            JVComm.setStatusMessage(this, 1, "Invalid configuration: " + 
                    cfg.getServer() + " : " + cfg.getPort());
            return false;
        }
        return true;
    }

    /** Register a JVisu variable with this instance
     * @param var JVisu variable, which has to be registered
     */    
    protected void RegisterVar(JVVar var) {
        RegisterServerVars (true);
    }
    
    /** Unregister a JVisu variable with this instance
     *  Stop communication, if all variables are unregistered
     * @param var JVisu variable, which has to be unregistered
     */    
    protected void UnRegisterVar(JVVar var) {
        UnRegisterServerVar(var);
    }

    /** Send a variable value to the server
     * @param var JVisu variable which has to be sent to the server
     * @return true on success
     */    
    public boolean SendVar(JVVar var) {
        if (!isConnected())
            return false;

        ByteBuffer bbuf = ByteBuffer.allocate (1024);
        bbuf.order (java.nio.ByteOrder.LITTLE_ENDIAN);
        bbuf.clear();
        bbuf.put (CMD_VISU_DATA);                           // command register variable
        bbuf.put ((byte) var.getVarCfg().getType());        // datatype
        bbuf.putShort ((short) (0));                        // length
        bbuf.putShort ((short) var.getSrvIndex());          // variable index
        bbuf.putShort ((short) var.getVarCfg().getSize());  // variable array size
        return SendData (var.GetVarValue(bbuf));
    }
    
    /** Get the activation status
     * @return true, if active
     */    
    public boolean isActive() {
        return isActive; 
    }
    
    /** Run the connection thread
     */
    public void run() {
        while (isActive) {
            DoCommunication ();
            // if we come back, wait 1 second before trying to reconnect
            try { Thread.sleep (1000); }
            catch (InterruptedException e) {break;}
        }
        CloseSocket();
    }    

    /** Start communication */
    protected void startComm() {
        commThread = new Thread(this);
        isActive = true;
        commThread.start();
        JVComm.setStatusMessage(this, 0, "Started");
    }
    
    /** Stop communication to server.
     *  This can only be done, if all variables are unregistered.
     */    
    protected boolean stopComm() {
        isActive = false;
        setConnected (false);
        try { commThread.interrupt(); }
        catch (Exception e) {};
        try { commThread.join(1000); }
        catch (Exception e) {};
        JVComm.setStatusMessage(this, 0, "Stopped");
        return true;
    }
    
    /** Do the communication stuff
     */
    private void DoCommunication () {
        if (socket == null) {
            if (commcfg.getServer() == null || commcfg.getServer().length() == 0)
                return;
            try {
                socket = SocketChannel.open();
                if (socket == null)
                    return;
                socket.connect (new InetSocketAddress(commcfg.getServer(), commcfg.getPort()));
            } catch (Exception eio) {
                JVComm.setStatusMessage(this, 1, "Error on connect:" + eio.toString());
                CloseSocket ();
                return;
            }
            JVComm.setStatusMessage(this, 0, "Connected");
        }
        setConnected (true);
        // Discard buffer
        recBuffer.clear();

        // Check, if variables have to be registered with the server
        RegisterServerVars (false);
        
        // Loop until we got all pending data from the socket
        while (true) {
            int read;
            try { 
                read = socket.read (recBuffer); 
            } catch (IOException eio) { 
                JVComm.setStatusMessage(this, 1, "Error on read:" + eio.toString());
                CloseSocket ();
                return;
            }
            if (read < 0) {
                CloseSocket ();
                return;
            }
            // no more data available ?
            if (read == 0)
                continue;
            AnalyseData ();
        }
    }
    
    /** Analyse the received data
     *  Actually it can only read variable data, but who knows ?
     */
    private void AnalyseData () {
        while (true) {
            int avail = recBuffer.position ();
            if (avail < HEADER_SIZE)                           // have we enough to read the header ?
                return;
            
            int len = (int) recBuffer.getShort (LENGTH_POS);   // read the length of this datagram
            if (avail < len)                                   // if we have not enough data to read the datagramm
                return;

            recBuffer.flip ();                                 // rewind the buffer and start from pos. 0
            // What kind of data have we got ?
            switch ( (int) recBuffer.get()) {
                case CMD_VISU_DATA: CheckVisuData (); break;
                default: break;
            }
            recBuffer.position (len);                          // set current position to datagram length
            recBuffer.compact ();                              // remove this datagramm from the buffer
            recBuffer.position (avail - len);                  // set the position to the end of the data in buffer
        }
    }

    /** CheckVisuData reads the incoming visudata depending on
     *  datatype and saves the received value in the corresponding
     *  Variable
     */
    private void CheckVisuData () {
        int type = (int) recBuffer.get ();
        int len = (int) recBuffer.getShort ();
        int idx = recBuffer.getInt ();
        JVVar v = GetVariable(idx);
        // Check, if it was removed in meantime
        if (v == null)
            return;
        v.SetVarValue(recBuffer);
    }

    /** Register variables with the server
     * @param force if true registration is forced independent of the state of reqest of registration
     */
    private void RegisterServerVars (boolean force) {
        if (socket == null || !socket.isConnected()) {
            reqRegistration |= force;
            return;
        }
        if (reqRegistration == false && force == false)
            return;
        
        for (int i = 0; i < vars.size(); i++) {
            JVVar v = (JVVar) vars.get(i);
            if (v.getSrvIndex() != -1 || v.getSymbol() == null)
                continue;
            int srvidx = registeredVars++;    
            ByteBuffer bbuf = ByteBuffer.allocate (256);
            bbuf.order (java.nio.ByteOrder.LITTLE_ENDIAN);
            bbuf.clear();
            bbuf.put (CMD_REGVAR);                              // command register variable
            bbuf.put ((byte) v.getVarCfg().getType());          // datatype
            bbuf.putShort ((short) (0));                        // length
            bbuf.putInt (srvidx);                               // variable index
            bbuf.putInt (v.getVarCfg().getRate());              // updaterate in ms
            bbuf.put (v.getSymbol().getBytes());                // Symbol of this variable
            if (SendData (bbuf) == false) {                     // Send it down there
                    v.setSrvIndex(-1);
                    return;
            }
            v.setSrvIndex(srvidx);
        }
        // Whee, we have managed to register all variables
        reqRegistration = false;
    }
    
    /** Unregister a variable 
     * @param var Variable, which has to be unregistered
     */
    private void UnRegisterServerVar (JVVar var) {
        ByteBuffer bbuf = ByteBuffer.allocate (VAR_HEADER_SIZE);
        bbuf.order (java.nio.ByteOrder.LITTLE_ENDIAN);
        bbuf.clear();
        bbuf.put (CMD_UNREGVAR);            // command unregister variable
        bbuf.put ((byte) 0);                // unused
        bbuf.putShort ((short) 0);          // length
        bbuf.putInt (var.getSrvIndex());    // variable index
        SendData (bbuf);                    // Send it down there
        var.setSrvIndex(-1);                // Invalidate variable
    }
    
    /** Close the socket connection
     */
    private void CloseSocket () {
        if (socket != null && socket.isConnected()) {
            try { 
                socket.close(); 
            } catch (IOException eio) { 
            }
        }
        socket = null;
        setConnected (false);
        // invalidate the registered variables
        for (int i = 0; i < vars.size(); i ++) {
            JVVar v = (JVVar) vars.get(i);
            v.setSrvIndex(-1);
        }
        reqRegistration = true;
    }
    
    /** SendData tries to send the given byte buffer to JVisu-Server
     *  Do not rewind or flip the ByteBuffer, as it is done here
     *  The length information is set in this function too
     *  @param buf buffer containing the telegram
     *  @return true on success
     */
    private boolean SendData (ByteBuffer buf) {
        buf.putShort(LENGTH_POS,(short) buf.position());  // Set the length
        //System.out.println(buf.position());
        buf.flip ();
        return SendSocketData (buf, this);
    }

    /** SendSocketData sends a datagram to the server
     * This method must be synchronized to avoid conflicts between all
     * clients, which are using it.
     * @param buf bytebuffer, which contains the data
     * @param instance instance of JVCommSock, which calls this method
     * @return true on success
     */
    private synchronized static boolean SendSocketData (ByteBuffer buf, JVSockComm instance) {
        if (instance.socket == null || !instance.socket.isConnected())
            return false;
        
        while (buf.remaining () > 0) {
            try { 
                instance.socket.write (buf); 
            } catch (IOException eio) { 
                JVComm.setStatusMessage(instance, 1, "Error on write:" + eio.toString());
            }
        }
        return true;
    }
}
