/*
 * JVPanel.java
 *
 * Created on 15. September 2003, 08:11
 */

package org.jvisu.panel;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.io.*;
import java.lang.reflect.*;
import java.text.*;
import java.util.zip.*;
import java.beans.*;
import javax.swing.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.jvisu.base.*;
import org.jvisu.util.param.JVParams;

/**
 *  <p>package:    org.jvisu.panel</p>
 *  <p>file:       JVPanel.java</p>
 * $Id: JVBasePanel.java 318 2007-08-21 11:23:19Z mol $
 *
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 * <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.</p>
 * </p>
 *
 * Change Log:
 * </p>
 * <ul>g
 * <li> 1.0 First implementation
 * <li> 1.2 Changes due to new variable and communication handling
 * <li> 1.3 Support absolute component placing with respect to the
 *          fact, that array based component creation must be possible.
 * </ul>
 * <p>
 * </p>
 *  The purpose of JVPanel is to provide a XML configurable component, which
 *  can hold one or more JVisu components and standard swing components like
 *  labels. Other components, which need user interaction should not be
 *  used, as at the moment no code can be associated with user actions.
 *
 *  The panel can be used to design complete subsets of an HMI by adding and
 *  layouting the interface with the JVPanelCfg application and store it in
 *  a XML file for runtime configuration.
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.3
 */
public class JVBasePanel extends JVPanel {
 
    /** the configuration file address. The address is 
     *  relative to base, if base is != null. Else it is
     *  treated as an absolute reference.
     */
    protected String                  configFile;
    
    /** Creates a new instance of JVPanel */
    public JVBasePanel() {
       this.autosize = false;
       setLayout(new SpringLayout());
    }
    
    /** Property change listener
     *  @param evt the property event
     */
    public void propertyChange(PropertyChangeEvent evt) {
        super.propertyChange(evt);
        String key = evt.getPropertyName();
        /* These are local events */
        if (evt.getSource() == this) {
            if (key.equals("base") || key.equals("configFile")) {
                parseFile();
            }
        }
    }
    
    /** Not used here */
    public void Shutdown() {
    }
    
    /** Not used here */
    public void StartUp() {
    }
    
    /** Not used here
     */
    public void VarChanged(org.jvisu.comm.var.JVVar var) {        
    }
    
    /** Get the address of the configuration file
     * @return configuration file address
     */
    public String getConfigFile() {
        return configFile;
    }
    
    /** Set the configfile address
     * @param configFile New configuration file
     */
    public void setConfigFile(String configFile) {
        String old = this.configFile;
        this.configFile = configFile;
        firePropertyChange("configFile", old, configFile);
    }

    /** Read a palette configuration file 
     */
    public void parseFile() {
        this.removeAll();
        this.setImageFile(null);
        this.setCommtype("org.jvisu.comm.JVSimComm");
        status = null;
        JVBasePanel.PanelHandler handler = new JVBasePanel.PanelHandler(this);
        if (configFile != null) {
            try {
                URL url;
                if (base == null)
                    url = new URL(configFile);
                else
                    url = new URL(new URL(base), configFile);
                URLConnection ctrlConnection = url.openConnection();
                InputStream in = ctrlConnection.getInputStream();
                if (configFile.endsWith(".gz"))
                    in = new GZIPInputStream(in);
                try {
                    SAXParser p = (SAXParserFactory.newInstance()).newSAXParser();
                    p.parse(in, handler);
                } catch (IOException e) {
                    status = "Error while reading configfile: " + configFile;
                    System.err.println(status);
                    System.err.println("Base: " + base);
                    System.err.println(e.toString());
                } catch (Exception e) {
                    status = "Error while reading configfile: " + configFile;
                    System.err.println(status);
                    System.err.println("Base: " + base);
                    System.err.println(e.toString());
                }
                in.close();
            
            } catch (Exception e) { 
                status = "Error while reading configfile:" + configFile;
                System.err.println(status);
                System.err.println("Base: " + base);
                System.err.println(e.toString());
            }
        }
        revalidate();
        repaint();
    }    
    
    /** Try toCreate a component 
     */
    public static synchronized JComponent createComponent(String name) {
        Object object = null;
        try {
            Class classDefinition = Class.forName(name);
            object = classDefinition.newInstance();
        } catch (InstantiationException e) {
            System.out.println(e);
        } catch (IllegalAccessException e) {
            System.out.println(e);
        } catch (ClassNotFoundException e) {
            System.out.println(e);
        }
        
        JComponent newcomp;
        try {
            newcomp = (JComponent) object;
        } catch (ClassCastException e) {
            return null;
        } catch (NullPointerException e) {
            return null;
        }
        return newcomp;
    }
    
    /** Set property of component.
     *  @param comp Component to parametrize
     *  @param name Property name
     *  @param value property value
     */
    public static synchronized void setProperty(JComponent comp, String name, String value) 
        throws ClassCastException, IllegalAccessException, NoSuchMethodException,
                InvocationTargetException {

        String prop = name.toUpperCase().substring(0,1) + name.substring(1);
        
        Method readm = null;
        try {
            readm = comp.getClass().getMethod(("get" + prop), (Class[])null);
        } catch (Exception e) {
            readm = comp.getClass().getMethod(("is" + prop), (Class[])null);
        }
        
        Class clazz[] = new Class[] {readm.getReturnType()};
        Constructor constr = null;
        Object[] args = null;
        try {
            /* Check, if a constructor of type string exists for this class */
            Class[] init = new Class[] {java.lang.String.class};
            constr = clazz[0].getConstructor(init);
            Object inst = constr.newInstance(new Object[] {value});
            args = new Object[] {inst};
        } catch (Exception e) {
            /* Try to find a basic property setter */
            Object basic = parseBasic(clazz[0], value, comp);
            args = new Object[] {basic};
        }
        comp.getClass().getMethod(("set" + prop), clazz).invoke(comp, args);
    }

    /** Set indexed property of component.
     *  @param comp Component to parametrize
     *  @param name Property name
     *  @param value property value
     *  @param index index of the property to set
     */
    public static synchronized void setIndexedProperty(JComponent comp, String name, String value, int index) 
        throws ClassCastException, IllegalAccessException, NoSuchMethodException,
                InvocationTargetException {

        String prop = name.toUpperCase().substring(0,1) + name.substring(1);
        
        Class[] rdargs = new Class[] {Integer.class};
        Method readm = null;
        try {
            readm = comp.getClass().getMethod(("get" + prop), rdargs);
        } catch (Exception e) {
            readm = comp.getClass().getMethod(("is" + prop), rdargs);
        }
        
        Class clazz[] = new Class[] {readm.getReturnType()};
        Constructor constr = null;
        Object[] args = null;
        try {
            /* Check, if a constructor of type string exists for this class */
            Class[] init = new Class[] {java.lang.String.class};
            constr = clazz[0].getConstructor(init);
            Object inst = constr.newInstance(new Object[] {value});
            args = new Object[] {inst, new Integer(index)};
        } catch (Exception e) {
            /* Try to find a basic property setter */
            Object basic = parseBasic(clazz[0], value, comp);
            args = new Object[] {basic, new Integer(index)};
        }
        comp.getClass().getMethod(("set" + prop), clazz).invoke(comp, args);
    }

    /** Parse basic parameter types and create a matching setter object if possible
     *  @param cl           class to inspect
     *  @param val          value which should be set
     *  @param component    object, which has to be parametrized
     *  @return object to set or null
     */
    private static Object parseBasic(Class cl, String val, JComponent comp) {
        
        try {
            if (cl.isAssignableFrom(Integer.class) || cl.equals(Integer.TYPE))
                return new Integer(val);
            
            else if (cl.isAssignableFrom(Long.class) || cl.equals(Long.TYPE))
                return new Long(val);
            
            else if (cl.isAssignableFrom(Short.class) || cl.equals(Short.TYPE))
                return new Short(val);
            
            else if (cl.isAssignableFrom(Byte.class) || cl.equals(Byte.TYPE))
                return new Byte(val);
            
            else if (cl.isAssignableFrom(Boolean.class) || cl.equals(Boolean.TYPE))
                return new Boolean(val);
                
            else if (cl.isAssignableFrom(Float.class) || cl.equals(Float.TYPE))
                return new Float(val);
            
            else if (cl.isAssignableFrom(Double.class) || cl.equals(Double.TYPE))
                return new Double(val);
            
            else if (cl.isAssignableFrom(java.awt.Color.class))
                return org.jvisu.util.param.JVParams.StringToColor(val, null);

            else if (cl.isAssignableFrom(java.awt.Dimension.class))
                return org.jvisu.util.param.JVParams.StringToDimension(val, null);

            else if (cl.isAssignableFrom(java.awt.Point.class))
                return org.jvisu.util.param.JVParams.StringToPoint(val, null);

            else if (cl.isAssignableFrom(java.awt.Font.class))
                return org.jvisu.util.param.JVParams.StringToFont(val, null);

        } catch (Exception e) { }
        return null;
    }
    
    /** The xml handler to read the panel xml
     */
    protected class PanelHandler extends org.xml.sax.helpers.DefaultHandler {

        /** The current indentation level */
        private int                 level = 0;
        /** The component to which stuff must be added */
        private JComponent          panel;
        /** The component, which is going to be added */
        private JComponent          comp;
        /** current qName */
        private String              qname;
        /** current property name for indexed properties */
        private String              property;
        /** Beaninfo */
        private BeanInfo            info;
        
        /** Creates a new instance of ItemHandler
         *  @param pan panel to add items
         */
        public PanelHandler(JComponent pan) {
            panel = pan;
            comp = panel;
        }
        
        /** Decode the content */
        private String Decode (String content) {
            try {
                return java.net.URLDecoder.decode(content, "UTF-8");
            } catch (Exception e) { 
                System.out.println(e);
                System.out.println(content);
            }
            return content;
        }
    
        public void startElement(String uri, String localName, String qName, Attributes attributes) {
            qname = qName;
            level++;
            /** Create a new item */
            if (qName.equals("visuitem")) {
                int idx = attributes.getIndex("object");
                if (idx != -1) {
                    String name = Decode(attributes.getValue(idx));
                    JComponent c = JVBasePanel.createComponent(name);
                    if (c != null) {
                        comp.add(c,0);
                        comp = c;
                        info = null;
                    }
                }
            } else if (qName.equals("property")) {
                int idx1 = attributes.getIndex("name");
                int idx2 = attributes.getIndex("value");
                int idx3 = attributes.getIndex("indexed");
                /* Indexed properties must be handled different */
                if (idx3 == -1 || !attributes.getValue(idx3).equals("true")) {
                    if (idx1 != -1 && idx2 != -1) {
                        String name = Decode(attributes.getValue(idx1));
                        if (name.equals("preferredSize"))
                            name = "designsize";
                        if (name.equals("font"))
                            name = "designFont";
                        try {
                            JVBasePanel.setProperty(comp, 
                                name, 
                                Decode(attributes.getValue(idx2)));
                        } catch (Exception e) {
                            System.err.println ("Could not set property: " +
                                name + ", " +
                                Decode(attributes.getValue(idx2)));
                        }
                    }
                } else {
                    if (idx1 != -1)
                        property = Decode(attributes.getValue(idx1));
                    else
                        property = null;
                }
            } else if (qName.equals("indexedvalue")) {
                int idx1 = attributes.getIndex("index");
                int idx2 = attributes.getIndex("value");
                if (idx1 != -1 && idx2 != -1 && property != null) {
                    try {
                        JVBasePanel.setIndexedProperty(comp, 
                                property, 
                                Decode(attributes.getValue(idx2)),
                                Integer.parseInt(attributes.getValue(idx1)));
                    } catch (Exception e) {
                            System.err.println ("Could not set property: " +
                                property + ", " +
                                Decode(attributes.getValue(idx2)));
                    }
                }
            } 
        }

        public void endElement(String uri, String localName, String qName) {
            level--;
            if (qName.equals("visuitem"))
                comp = (JComponent) comp.getParent();
        }
        
        public void processingInstruction(String target, String data) {
            System.out.println("processing instruction: " + target);
        }

        public void characters(char[] ch, int start, int length) {
            // String c = Decode(new String(ch, start, length));
        }
    
        public void warning(SAXParseException e) {
        System.out.println("Warning:" + (e.getMessage () != null
                    ? e.getMessage ()
                    : e.getClass ().getName ()));
        }
    
        public void error(SAXParseException e) {
        System.out.println("Error:" + (e.getMessage () != null
                    ? e.getMessage ()
                    : e.getClass ().getName ()));
        }    
    }        

    /** Main method for standalone testing */
     public static void main(String args[]) {
        final JFrame frame = new JFrame();
        frame.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                System.exit(0);
            }
        });
        frame.getContentPane().setLayout(new BoxLayout(frame.getContentPane(), BoxLayout.Y_AXIS));
        
        final JVBasePanel pan = new JVBasePanel();
        pan.setServer("localhost");
        //pan.setCommtype("org.jvisu.comm.JVSockComm");
        pan.setBase(("file:///" + System.getProperty("user.dir") + "/bla"));
        if (args.length > 0)
            pan.setConfigFile(args[0]);
        else
            pan.status = "No configfile in commandline";
        pan.setEditEnabled(true);
        
        //pan.zoomfactor = 0.5;
        
        final JButton b = new JButton("2:1");
        b.addActionListener(new java.awt.event.ActionListener () {
            public void actionPerformed(java.awt.event.ActionEvent e) {
                pan.setZoomfactor((pan.getZoomfactor() == 1.0) ? 2.0 : 1.0);
                b.setText(pan.getZoomfactor() == 1.0 ? "2:1" : "1:1");
                frame.pack();
            }
        });
        frame.getContentPane().add(b);
        
        frame.getContentPane().add(pan);
        frame.pack();
        frame.setVisible(true);
    }        
}
