package org.jvisu.panel.configurator.events;

import java.io.*;
import javax.swing.*;

/** Editor events for undo / redo
 *
 *  <p>$Id: JVEditEvent.java 228 2003-11-22 13:05:34Z tgl $
 *  </p>
 *  <p>Copyright 2003 Thomas Gleixner <tglx@linutronix.de></p>
 *
 *  <p>This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation. See file LICENCE in this
 *  directory</p>
 *
 *  <p>Change Log:</p>
 *  <ul>
 *  <li> 1.0 Initial inmplimentation </li>
 *  </ul>
 *  <p>
 *  <p>Editor event for undo / redo with convenience classes for storing
 *     the event information depending on the event type.
 *  </p>
 *
 * @author Thomas Gleixner <tglx@linutronix.de>
 * @version 1.0
 */
public class JVEditEvent extends Object {
    
    /** Event type Component added */
    public static final int COMP_ADDED = 1;
    /** Event type Component removed */
    public static final int COMP_REMOVED = 2;
    /** Event type Property changed */
    public static final int PROP_CHANGED = 3;
    /** Event type component selected */
    public static final int COMP_SELECTED = 4;
    /** Event type component selected */
    public static final int COMP_CLICKED = 5;
    /** Event type start bulk */
    public static final int BULK_START = 6;
    /** Event type end bulk */
    public static final int BULK_END = 7;
    
    /** The event type */
    private int                 type;
    /** The source for this event */
    private Object              source;
    /** The object for this event */
    private Object              object;
    
    /** Create a new instance of JVEditEvent
     *  @param typ the event source
     *  @param typ the event type
     *  @param obj An object associated with the event
     */
    public JVEditEvent(Object source, int typ, Object obj) {
        this.source = source;
        type = typ;
        object = obj;
    }

    /** Create a new instance of JVEditEvent for a add event
     *  @param typ the event type
     *  @param par the parent component
     *  @param child the added component
     */
    public JVEditEvent(Object source, int typ, JComponent par, JComponent child) {
        this.source = source;
        type = typ;
        object = new JVEditEvent.CompAdd(par,child);
    }

    /** Create a new instance of JVEditEvent for a remove event
     *  @param typ the event source
     *  @param typ the event type
     *  @param par the parent component
     *  @param xml the Stringreader, which contains the XML description
     *              of the removed component(s)
     */
    public JVEditEvent(Object source, int typ, JComponent par, StringReader xml) {
        this.source = source;
        type = typ;
        object = new JVEditEvent.CompXML(par, xml);
    }

    /** Create a new instance of JVEditEvent for a property change event
     *  @param typ the event source
     *  @param typ the event type
     *  @param comp the component, whose property was changed
     *  @param name The name of the property
     *  @param value The previous value of the property
     */
    public JVEditEvent(Object source, int typ, JComponent comp, String name, String value) {
        this.source = source;
        type = typ;
        object = new JVEditEvent.PropertyChange(comp, name, value);
    }
    
    /** Get the event type
     * @return event type
     */
    public int getType() {
        return type;
    }

    /** Set the event type
     *  @param typ the new event type
     */
    public void setType(int typ) {
        type = typ;
    }

    /** Get the event source
     * @return the event source
     */
    public Object getSource() {
        return source;
    }    

    /** Return the object for this event
     *  @return the object fr this event
     */
    public Object getObject() {
        return object;
    }
    
    /** A convenience class for component add events
     */
    public class CompAdd extends Object {
        
        /** The parent of the newly added component */
        private JComponent   parent;
        /** The newly added component */
        private JComponent   child;
        
        /** Create a new instance of CompAdd
         *  @param par the parent object of this event
         *  @param comp the designcomponent, which was added
         */
        public CompAdd(JComponent par, JComponent comp) {
            parent = par;
            child = comp;
        }
        
        /** Return the parent of this event
         *  @return the parent
         */
        public JComponent getParent() {
            return parent;
        }
        
        /** Return the child of this event
         *  @return the child
         */
        public JComponent getChild() {
            return child;
        }
    }
    
    /** A convenience class for component remove events
     */
    public class CompXML extends Object {
        
        /** The parent of the removed component */
        JComponent   parent;
        /** The Stringreader, which contains the XML description */
        StringReader    desc;
        
        /** Create a new instance of CompAdd
         *  @param par the parent object of this event
         *  @param xml the Stringreader, which contains the XML description
         *              of the removed component(s)
         */
        public CompXML(JComponent par, StringReader xml) {
            parent = par;
            desc = xml;
        }
        
        /** Return the parent of this event
         *  @return the parent
         */
        public JComponent getParent() {
            return parent;
        }
        
        /** Return the xml description of the removed component
         *  @return the xml description of the removed component
         */
        public StringReader getXMLDescription() {
            return desc;
        }
        
    }
    
    /** A convenience class for property change events
     */
    public class PropertyChange extends Object {
        
        /** The component, whose property was changed */
        JComponent      component;
        /** The name of the property */
        String          name;
        /** The value of the property */
        String          value;
        
        /** Create a new instance of PropertyChange
         *  @param comp The component, whose property was changed
         *  @param name The name of the property
         *  @param value The value of the property
         */
        public PropertyChange(JComponent comp, String name, String value) {
            component = comp;
            this.name = name;
            this.value = value;
            //System.out.println(name + ": " + value);
        }
        
        /** Return the component, whose property was changed
         *  @return the component, whose property was changed
         */
        public JComponent getComponent() {
            return component;
        }
        
        /** Return the name of the property
         *  @return the name of the property
         */
        public String getName() {
            return name;
        }
        
        /** Return the value of the property
         *  @return the value of the property
         */
        public String getValue() {
            return value;
        }
    }
}
